"""
footer.py
Lines to be displayed on the help screen.

Author: GrimAndGreedy
License: MIT
"""

import curses
import logging

logger = logging.getLogger('picker_log')

class Footer:
    def __init__(self, stdscr, colours_start, get_state_function):
        """
        stdscr: curses screen object
        colours_start: base colour pair index
        get_state_callback: function that returns a dict with all required data for rendering
        """
        self.stdscr = stdscr
        self.colours_start = colours_start
        self.get_state = get_state_function
        self.height = 0

    def draw(self, h, w):
        """
        Draw the footer. Must be implemented by subclasses.
        """
        raise NotImplementedError

class StandardFooter(Footer):
    def __init__(self, stdscr, colours_start, get_state_function):
        """
        stdscr: curses screen object
        colours_start: base colour pair index
        get_state_callback: function that returns a dict with all required data for rendering
        """
        self.stdscr = stdscr
        self.colours_start = colours_start
        self.get_state = get_state_function

        self.height = 2
        try:
            state = self.get_state()
            if "footer_string" in state and state["footer_string"]: self.height = 3
            else: self.height = 2
        except:
            logger.error("Error encountered when running StandardFooter.get_state")

        self.footer_string_y = 0
        self.picker_info_y = 1
        self.sort_info_y = 2
        self.sheets_y = 3
        self.files_y = 4
    def adjust_sizes(self, h, w):
        state = self.get_state()

        self.sheets_y=-1
        if state["footer_string"]:
            self.footer_string_y = h-1
            self.picker_info_y = h-3
            self.sort_info_y = h-2
            
            self.height = 3

        else:
            self.picker_info_y = h-2
            self.sort_info_y = h-1
            self.footer_string_y = -1
            self.height = 2

        if len(state["sheets"]) > 1:
            self.height += 1
            self.picker_info_y -= 1
            self.sort_info_y -= 1
            self.footer_string_y -= 1
            self.sheets_y = h-1

        if len(state["loaded_files"]) > 1 and state["loaded_file"] in state["loaded_files"]:
            self.height += 1
            self.picker_info_y -= 1
            self.sort_info_y -= 1
            self.footer_string_y -= 1
            self.sheets_y -= 1

            self.files_y = h-1
        if state["search_query"] and self.height < 3:
            self.height = 3


    def draw(self, h, w):
        state = self.get_state()
        # Fill background

        self.adjust_sizes(h, w)



        ## Clear background of footer rows
        for i in range(self.height):
            self.stdscr.addstr(h-self.height+i, 0, ' '*(w-1), curses.color_pair(self.colours_start+20))

        # Display loaded files
        if len(state["loaded_files"]) > 1 and state["loaded_file"] in state["loaded_files"]:

            sep = "◢ "
            files = [x.split("/")[-1] for x in state["loaded_files"]]
            filename = state["loaded_file"].split("/")[:-1]

            files_str = sep.join(files)
            files_str = files_str[:w-2]

            idx = state["loaded_file_index"]
            current_file_x = sum((len(x) for x in files[:idx])) + idx*len(sep)
            current_file_str = state["loaded_file"].split("/")[-1]
            current_file_x_end = current_file_x + len(current_file_str) + 2
            self.stdscr.addstr(self.files_y, 0, ' '*(w-1), curses.color_pair(self.colours_start+4))
            if current_file_x_end < w:

                self.stdscr.addstr(self.files_y, 0, f" {files_str}", curses.color_pair(self.colours_start+4))

                self.stdscr.addstr(self.files_y, current_file_x, f" {current_file_str}{sep[0]}", curses.color_pair(self.colours_start+4) | curses.A_REVERSE)
            else:
                files_str = sep.join(files)
                files_str = files_str[current_file_x_end-w:current_file_x_end][:w-2]
                self.stdscr.addstr(self.files_y, 0, f" {files_str}", curses.color_pair(self.colours_start+4))

                self.stdscr.addstr(self.files_y, w - (len(current_file_str)+3), f" {current_file_str}{sep[0]}", curses.color_pair(self.colours_start+4) | curses.A_REVERSE)

        if len(state["sheets"]) > 1:

            sep = "◢ "
            sheets = [x.split("/")[-1] for x in state["sheets"]]
            filename = state["sheet_name"].split("/")[:-1]

            sheets_str = sep.join(sheets)
            sheets_str = sheets_str[:w-2]

            idx = state["sheet_index"]
            current_sheet_x = sum((len(x) for x in sheets[:idx])) + idx*len(sep)
            current_sheet_str = state["sheet_name"].split("/")[-1]
            current_sheet_x_end = current_sheet_x + len(current_sheet_str) + 2
            self.stdscr.addstr(self.sheets_y, 0, ' '*(w-1), curses.color_pair(self.colours_start+4))
            if current_sheet_x_end < w:

                self.stdscr.addstr(self.sheets_y, 0, f" {sheets_str}", curses.color_pair(self.colours_start+4))

                self.stdscr.addstr(self.sheets_y, current_sheet_x, f" {current_sheet_str}{sep[0]}", curses.color_pair(self.colours_start+4) | curses.A_REVERSE)
            else:
                sheets_str = sep.join(sheets)
                sheets_str = sheets_str[current_sheet_x_end-w:current_sheet_x_end][:w-2]
                self.stdscr.addstr(self.sheets_y, 0, f" {sheets_str}", curses.color_pair(self.colours_start+4))

                self.stdscr.addstr(self.sheets_y, w - (len(current_sheet_str)+3), f" {current_sheet_str}{sep[0]}", curses.color_pair(self.colours_start+4) | curses.A_REVERSE)
                



        if state["footer_string"]:
            footer_string_width = min(w-1, len(state["footer_string"])+2)

            disp_string = f"{state["footer_string"][:footer_string_width]}"
            disp_string = f" {disp_string:>{footer_string_width-2}} "
            self.stdscr.addstr(self.footer_string_y, w-footer_string_width-1, " "*footer_string_width, curses.color_pair(self.colours_start+24))
            self.stdscr.addstr(self.footer_string_y, w-footer_string_width-1, f"{disp_string}", curses.color_pair(self.colours_start+24))

        


        if state["filter_query"]:
            self.stdscr.addstr(h - 2, 2, f" Filter: {state['filter_query']} "[:w-40], curses.color_pair(self.colours_start+20) | curses.A_BOLD)
        if state["search_query"]:
            self.stdscr.addstr(h - 3, 2, f" Search: {state['search_query']} [{state['search_index']}/{state['search_count']}] "[:w-3], curses.color_pair(self.colours_start+20) | curses.A_BOLD)
        if state["user_opts"]:
            self.stdscr.addstr(h - 1, 2, f" Opts: {state['user_opts']} "[:w-3], curses.color_pair(self.colours_start+20) | curses.A_BOLD)




        ## Cursor selection mode
        select_mode = "C"
        if state["is_selecting"]: select_mode = "VS"
        elif state["is_deselecting"]: select_mode = "VDS"
        if state["pin_cursor"]: select_mode = f"{select_mode} "

        # Cursor & selection info
        selected_count = sum(state["selections"].values())
        if state["paginate"]:
            cursor_disp_str = f" [{selected_count}] {state['cursor_pos']+1}/{len(state['indexed_items'])} | Page {state['cursor_pos']//state['items_per_page']}/{len(state['indexed_items'])//state['items_per_page']} | {select_mode}"
        else:
            # cursor_disp_str = f" [{selected_count}] {state['cursor_pos']+1},{state['selected_column']}/{len(state['indexed_items'])},{len(state['column_widths'])} | {select_mode}"
            if state["cell_cursor"]:
                cursor_disp_str = f" [{selected_count}] {state['cursor_pos']+1}/{len(state['indexed_items'])} | {state['selected_column']}/{len(state['column_widths'])} | {select_mode}"
            else:
                cursor_disp_str = f" [{selected_count}] {state['cursor_pos']+1}/{len(state['indexed_items'])} | {select_mode}"

        # Maximum chars that should be displayed
        max_chars = min(len(cursor_disp_str)+2, w)
        self.stdscr.addstr(self.picker_info_y, w-max_chars, f"{cursor_disp_str:>{max_chars-2}} ", curses.color_pair(self.colours_start+20))


        # Sort info
        sort_column_info = f"{state['sort_column'] if state['sort_column'] is not None else 'None'}"
        sort_method_info = f"{state['SORT_METHODS'][state['columns_sort_method'][state['sort_column']]]}" if state['sort_column'] is not None else "NA"
        sort_order_info = "Desc." if state["sort_reverse"] else "Asc."
        sort_order_info = "▼" if state["sort_reverse"][state['sort_column']] else "▲"
        sort_disp_str = f" Sort: ({sort_column_info}, {sort_method_info}, {sort_order_info}) "
        max_chars = min(len(sort_disp_str)+2, w)
        self.stdscr.addstr(self.sort_info_y, w-max_chars, f"{sort_disp_str:>{max_chars-1}}", curses.color_pair(self.colours_start+20))

        self.stdscr.refresh()



class CompactFooter(Footer):
    def __init__(self, stdscr, colours_start, get_state_function):
        """
        stdscr: curses screen object
        colours_start: base colour pair index
        get_state_callback: function that returns a dict with all required data for rendering
        """
        self.stdscr = stdscr
        self.colours_start = colours_start
        self.get_state = get_state_function
        self.height = 1

    def adjust_sizes(self, h, w):
        pass

    def draw(self, h, w):
        state = self.get_state()

        # Fill background
        if state["search_query"]: self.height = 3
        elif state["filter_query"]: self.height = 2
        elif state["user_opts"]: self.height = 1
        elif state["footer_string"]: self.height = 2
        else: self.height = 1
        for i in range(self.height):
            self.stdscr.addstr(h-(i+1), 0, ' '*(w-1), curses.color_pair(self.colours_start+20))

        if state["user_opts"]:
            self.stdscr.addstr(h - 1, 2, f" Opts: {state['user_opts']} "[:w-3], curses.color_pair(self.colours_start+20) | curses.A_BOLD)
        if state["filter_query"]:
            self.stdscr.addstr(h - 2, 2, f" Filter: {state['filter_query']} "[:w-40], curses.color_pair(self.colours_start+20) | curses.A_BOLD)
        if state["search_query"]:
            self.stdscr.addstr(h - 3, 2, f" Search: {state['search_query']} [{state['search_index']}/{state['search_count']}] "[:w-3], curses.color_pair(self.colours_start+20) | curses.A_BOLD)

        right_width = 40
        # Sort info
        sort_column_info = f"{state['sort_column'] if state['sort_column'] is not None else 'None'}"
        sort_method_info = f"{state['SORT_METHODS'][state['columns_sort_method'][state['sort_column']]]}" if state['sort_column'] is not None else "NA"
        sort_order_info = "Desc." if state["sort_reverse"][state['sort_column']] else "Asc."
        sort_order_info = "▼" if state["sort_reverse"][state['sort_column']] else "▲"
        sort_disp_str = f" ({sort_column_info}, {sort_method_info}, {sort_order_info}) "
        # self.stdscr.addstr(h - 2, w-right_width, f"{sort_disp_str:>{right_width-1}}", curses.color_pair(self.colours_start+20))

        if state["footer_string"]:
            footer_string_width = min(w-1, len(state["footer_string"])+2)

            disp_string = f"{state["footer_string"][:footer_string_width]}"
            disp_string = f" {disp_string:>{footer_string_width-2}} "
            self.stdscr.addstr(h - 1, w-footer_string_width-1, " "*footer_string_width, curses.color_pair(self.colours_start+24))
            self.stdscr.addstr(h - 1, w-footer_string_width-1, f"{disp_string}", curses.color_pair(self.colours_start+24))
            selected_count = sum(state["selections"].values())
            if state["paginate"]:
                cursor_disp_str = f" {state['cursor_pos']+1}/{len(state['indexed_items'])}  Page {state['cursor_pos']//state['items_per_page']}/{len(state['indexed_items'])}  Selected {selected_count}"
            else:
                cursor_disp_str = f"{sort_disp_str} [{selected_count}] {state['cursor_pos']+1}/{len(state['indexed_items'])}"
            self.stdscr.addstr(h-2, w-right_width, f"{cursor_disp_str:>{right_width-2}}"[:right_width-1], curses.color_pair(self.colours_start+20))
        else:
            # Cursor & selection info
            selected_count = sum(state["selections"].values())
            if state["paginate"]:
                cursor_disp_str = f" {state['cursor_pos']+1}/{len(state['indexed_items'])}  Page {state['cursor_pos']//state['items_per_page']}/{len(state['indexed_items'])}  Selected {selected_count}"
            else:
                cursor_disp_str = f"{sort_disp_str} [{selected_count}] {state['cursor_pos']+1}/{len(state['indexed_items'])}"
            self.stdscr.addstr(h - 1, w-right_width, f"{cursor_disp_str:>{right_width-2}}"[:right_width-1], curses.color_pair(self.colours_start+20))

        self.stdscr.refresh()

class NoFooter(Footer):
    def __init__(self, stdscr, colours_start, get_state_function):
        """
        stdscr: curses screen object
        colours_start: base colour pair index
        get_state_callback: function that returns a dict with all required data for rendering
        """
        self.stdscr = stdscr
        self.colours_start = colours_start
        self.get_state = get_state_function
        self.height = 0

    def adjust_sizes(self, h, w):
        pass

    def draw(self, h, w):
        state = self.get_state()

        if state["search_query"]: self.height = 3
        elif state["filter_query"]: self.height = 2
        elif state["user_opts"]: self.height = 1
        elif state["footer_string"]: self.height = 1
        else: self.height = 0

        for i in range(self.height):
            self.stdscr.addstr(h-(i+1), 0, ' '*(w-1), curses.color_pair(self.colours_start+20))

        if state["user_opts"]:
            self.stdscr.addstr(h - 1, 2, f" Opts: {state['user_opts']} "[:w-3], curses.color_pair(self.colours_start+20) | curses.A_BOLD)
        if state["filter_query"]:
            self.stdscr.addstr(h - 2, 2, f" Filter: {state['filter_query']} "[:w-40], curses.color_pair(self.colours_start+20) | curses.A_BOLD)
        if state["search_query"]:
            self.stdscr.addstr(h - 3, 2, f" Search: {state['search_query']} [{state['search_index']}/{state['search_count']}] "[:w-3], curses.color_pair(self.colours_start+20) | curses.A_BOLD)
            self.height = 3


        if state["footer_string"]:
            footer_string_width = min(w-1, len(state["footer_string"])+2)
            disp_string = f"{state["footer_string"][:footer_string_width]}"
            disp_string = f" {disp_string:>{footer_string_width-2}} "
            self.stdscr.addstr(h - 1, w-footer_string_width-1, " "*footer_string_width, curses.color_pair(self.colours_start+24))
            self.stdscr.addstr(h - 1, w-footer_string_width-1, f"{disp_string}", curses.color_pair(self.colours_start+24))



#!/bin/python
# -*- coding: utf-8 -*-
"""
Get a wikipedia table and display it in a Picker.

Author: GrimAndGreedy
License: MIT
"""

import requests
from bs4 import BeautifulSoup
import pickle
from listpick.listpick_app import Picker, start_curses, close_curses


def fetch_and_parse_webpage(url: str) -> list:
    """ Get list of tables from webpage at url. """
    response = requests.get(url)
    if response.status_code == 200:
        soup = BeautifulSoup(response.content, 'html.parser')
        tables = soup.find_all('table')
        return [parse_table(table) for table in tables]
    else:
        raise Exception(f"Failed to fetch webpage: {response.status_code}")

def parse_table(table) -> list[list[str]]:
    """ Convert a bs4 table to a list of lists. """
    headers = [header.text.strip() for header in table.find('tr').find_all('th')]
    rows = []
    for row in table.find_all('tr')[1:]:
        cells = [cell.text.strip() for cell in row.find_all(['td', 'th'])]
        rows.append(cells)
    return [headers] + rows

def save_data(data, filename: str) -> None:
    """ Pickle data. """
    with open(filename, 'wb') as file:
        pickle.dump(data, file)

def load_data(filename: str, url: str) -> list:
    """ 
    Load tables from wikipedia page. Data is saved to `filename`. 
    If the tables have already been cached then we load the data and nothing is downloaded.
    """
    try:
        with open(filename, 'rb') as file:
            data = pickle.load(file)
        return data
    except FileNotFoundError:
        print(f"File not found. Fetching data from the web.")
        data = fetch_and_parse_webpage(url)
        save_data(data, filename)
        return data

if __name__ == "__main__":
    url = 'https://en.wikipedia.org/wiki/2024-25_Premier_League'

    # Filename for pickled data
    # pickle_filename = './auxiallary_files/premier_league_tables_2024.pkl'
    pickle_filename = f'./auxiallary_files/{url.split("/")[-1]}.pkl'
    try:
        tables = load_data(pickle_filename, url)
        league_table_number = 4
        items = tables[league_table_number][1:]
        header = tables[league_table_number][0]
        stdscr = start_curses()
        app = Picker(
            stdscr, 
            items=items, 
            header=header,
            title="Premier League Table 2024-25",
            colour_theme_number=3,
        )
        app.run()

        close_curses(stdscr)
    except:
        pass



#!/bin/python
# -*- coding: utf-8 -*-
"""
clipboard_operations.py
Copy selected items in selected format to the clipboard.

Author: GrimAndGreedy
License: MIT
"""

import pyperclip
from typing import Tuple
from listpick.utils.utils import get_selected_cells_by_row
import logging

logger = logging.getLogger('picker_log')

"""
representation: python, tab-separated, comma-separated, current view
rows: selected, all, filtered
columns: all, not hidden
"""

def copy_to_clipboard(
    items: list[list[str]],
    indexed_items: list[Tuple[int, list[str]]],
    selections: dict,
    cell_selections: dict,
    hidden_columns: set,
    representation: str="python",
    copy_hidden_cols: bool = False,
    separator="\t",
    cellwise:bool = False,

) -> None:
    """ 
    Copy selected items to clipboard.

    representation (str): The representation of the rows that should be copied.
                            accepted values: python, csv, tsv, current view, custom_sv

    """
    logger.info("function: copy_to_clipboard (clipboard_operations.py)")
    formatted_items = []
    if cellwise:
        if len(items):
            row_len = len(items[0])
            selected_cells_by_row = get_selected_cells_by_row(cell_selections)
            starty = min(selected_cells_by_row.keys())
            endy = max(selected_cells_by_row.keys())
            startx = min([val for row in selected_cells_by_row.values() for val in row])
            endx = max([val for row in selected_cells_by_row.values() for val in row])
            # for row_num in selected_cells_by_row.keys():
            for row_num in range(starty, endy+1):
                if representation == "python":
                    row = [None for i in range(startx, endx+1)]
                else:
                    row = ["" for i in range(startx, endx+1)]

                if row_num in selected_cells_by_row:
                    for cell_num in selected_cells_by_row[row_num]:
                        row[cell_num-startx] = items[row_num][cell_num]

                formatted_items.append(row)
    else:
        selected_indices = [i for i, selected in selections.items() if selected]
        rows_to_copy = [item for i, item in enumerate(items) if i in selected_indices]
        formatted_items = [[cell for i, cell in enumerate(item) if i not in hidden_columns or copy_hidden_cols] for item in rows_to_copy]

    if representation == "python":
        pyperclip.copy(repr(formatted_items))
    elif representation == "tsv":
        pyperclip.copy("\n".join(["\t".join(row) for row in formatted_items]))
    elif representation == "csv":
        pyperclip.copy("\n".join([",".join(row) for row in formatted_items]))
    elif representation == "custom_sv":
        # Ensure that escapes are interpreted properly in separator
        separator = bytes(separator, "utf-8").decode("unicode_escape")
        pyperclip.copy("\n".join([separator.join(row) for row in formatted_items]))
    # elif representation == "current_view":
    #     pyperclip.copy("\n".join([format_row(row, hidden_columns, column_widths, separator, centre_in_cols) for row in formatted_items]))



#!/bin/python
# -*- coding: utf-8 -*-
"""
setup.py

Author: GrimAndGreedy
Created: 2025-06-25
License: MIT
"""

import setuptools


with open("README.md", "r", encoding = "utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name = "listpick",
    version = "0.1.14.13",
    author = "Grim",
    author_email = "grimandgreedy@protonmail.com",
    description = "Listpick is a powerful TUI data tool for creating TUI apps or viewing/comparing tabulated data.",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = "https://github.com/grimandgreedy/listpick",
    project_urls = {
        "Bug Tracker": "https://github.com/grimandgreedy/listpick/issues",
    },
    classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
    ],
    package_dir = {"": "src"},
    packages = setuptools.find_packages(where="src"),
    python_requires = ">=3.9",

    entry_points={
        'console_scripts': [
            'listpick = listpick:main',
        ]
    },
    install_requires = [
        "wcwidth",
        "pyperclip",
        "toml",
        "dill",
    ],

    extras_require={
        "full": [
            "dill",
            "wcwidth",
            "ipython",
            "msgpack",
            "openpyxl",
            "pandas",
            # "pyarrow",
            "pyperclip",
            "toml",
            "traitlets",
            "odfpy",
        ]
    },

)



#!/bin/python
# -*- coding: utf-8 -*-
"""
Parse command-line arguments and run Picker.

Author: GrimAndGreedy
License: MIT
"""


from src.listpick.listpick_app import main
from src.listpick.ui.help_screen import *
from src.listpick.ui.input_field import *
from src.listpick.ui.keys import *
from src.listpick.ui.picker_colours import *
from src.listpick.utils.clipboard_operations import *
from src.listpick.utils.dump import *
from src.listpick.utils.filtering import *
from src.listpick.utils.generate_data import *
from src.listpick.utils.options_selectors import *
from src.listpick.utils.search_and_filter_utils import *
from src.listpick.utils.searching import *
from src.listpick.utils.sorting import *
from src.listpick.utils.table_to_list_of_lists import *
from src.listpick.utils.utils import *
from src.listpick.listpick_app import *


if __name__ == "__main__":
    main()
